@@ -15,29 +15,36 @@
 
 """Volume v1 Type action implementations"""
 
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
+from openstackclient.i18n import _
+
+
+LOG = logging.getLogger(__name__)
 
 
 class CreateVolumeType(command.ShowOne):
-    """Create new volume type"""
+    _description = _("Create new volume type")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolumeType, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<name>',
-            help='New volume type name',
+            help=_('Volume type name'),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Property to add for this volume type '
-                 '(repeat option to set multiple properties)',
+            help=_('Set a property on this volume type '
+                   '(repeat option to set multiple properties)'),
         )
         return parser
 
@@ -48,33 +55,49 @@ class CreateVolumeType(command.ShowOne):
         if parsed_args.property:
             result = volume_type.set_keys(parsed_args.property)
             volume_type._info.update({'properties': utils.format_dict(result)})
+        volume_type._info.pop("os-volume-type-access:is_public", None)
 
-        info = {}
-        info.update(volume_type._info)
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(six.iteritems(volume_type._info)))
 
 
 class DeleteVolumeType(command.Command):
-    """Delete volume type"""
+    _description = _("Delete volume type(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolumeType, self).get_parser(prog_name)
         parser.add_argument(
-            'volume_type',
+            'volume_types',
             metavar='<volume-type>',
-            help='Volume type to delete (name or ID)',
+            nargs='+',
+            help=_('Volume type(s) to delete (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         volume_client = self.app.client_manager.volume
-        volume_type_id = utils.find_resource(
-            volume_client.volume_types, parsed_args.volume_type).id
-        volume_client.volume_types.delete(volume_type_id)
+        result = 0
+
+        for volume_type in parsed_args.volume_types:
+            try:
+                vol_type = utils.find_resource(volume_client.volume_types,
+                                               volume_type)
+
+                volume_client.volume_types.delete(vol_type)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete volume type with "
+                            "name or ID '%(volume_type)s': %(e)s")
+                          % {'volume_type': volume_type, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.volume_types)
+            msg = (_("%(result)s of %(total)s volume types failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListVolumeType(command.Lister):
-    """List volume types"""
+    _description = _("List volume types")
 
     def get_parser(self, prog_name):
         parser = super(ListVolumeType, self).get_parser(prog_name)
@@ -82,7 +105,8 @@ class ListVolumeType(command.Lister):
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output')
+            help=_('List additional fields in output')
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -101,21 +125,21 @@ class ListVolumeType(command.Lister):
 
 
 class SetVolumeType(command.Command):
-    """Set volume type properties"""
+    _description = _("Set volume type properties")
 
     def get_parser(self, prog_name):
         parser = super(SetVolumeType, self).get_parser(prog_name)
         parser.add_argument(
             'volume_type',
             metavar='<volume-type>',
-            help='Volume type to modify (name or ID)',
+            help=_('Volume type to modify (name or ID)'),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Property to add or modify for this volume type '
-                 '(repeat option to set multiple properties)',
+            help=_('Set a property on this volume type '
+                   '(repeat option to set multiple properties)'),
         )
         return parser
 
@@ -128,24 +152,44 @@ class SetVolumeType(command.Command):
             volume_type.set_keys(parsed_args.property)
 
 
+class ShowVolumeType(command.ShowOne):
+    _description = _("Display volume type details")
+
+    def get_parser(self, prog_name):
+        parser = super(ShowVolumeType, self).get_parser(prog_name)
+        parser.add_argument(
+            "volume_type",
+            metavar="<volume-type>",
+            help=_("Volume type to display (name or ID)")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
+        volume_type = utils.find_resource(
+            volume_client.volume_types, parsed_args.volume_type)
+        properties = utils.format_dict(volume_type._info.pop('extra_specs'))
+        volume_type._info.update({'properties': properties})
+        volume_type._info.pop("os-volume-type-access:is_public", None)
+        return zip(*sorted(six.iteritems(volume_type._info)))
+
+
 class UnsetVolumeType(command.Command):
-    """Unset volume type properties"""
+    _description = _("Unset volume type properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetVolumeType, self).get_parser(prog_name)
         parser.add_argument(
             'volume_type',
             metavar='<volume-type>',
-            help='Volume type to modify (name or ID)',
+            help=_('Volume type to modify (name or ID)'),
         )
         parser.add_argument(
             '--property',
             metavar='<key>',
             action='append',
-            default=[],
-            help='Property to remove from volume type '
-                 '(repeat option to remove multiple properties)',
-            required=True,
+            help=_('Remove a property from this volume type '
+                   '(repeat option to remove multiple properties)'),
         )
         return parser
 
@@ -158,26 +202,3 @@ class UnsetVolumeType(command.Command):
 
         if parsed_args.property:
             volume_type.unset_keys(parsed_args.property)
-        else:
-            self.app.log.error("No changes requested\n")
-
-
-class ShowVolumeType(command.ShowOne):
-    """Display volume type details"""
-
-    def get_parser(self, prog_name):
-        parser = super(ShowVolumeType, self).get_parser(prog_name)
-        parser.add_argument(
-            "volume_type",
-            metavar="<volume-type>",
-            help="Volume type to display (name or ID)"
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        volume_client = self.app.client_manager.volume
-        volume_type = utils.find_resource(
-            volume_client.volume_types, parsed_args.volume_type)
-        properties = utils.format_dict(volume_type._info.pop('extra_specs'))
-        volume_type._info.update({'properties': properties})
-        return zip(*sorted(six.iteritems(volume_type._info)))
